# Author:: Gerhard Spitzlsperger
# EMAIL and WWW:: ruby@gs68.de, www.gs68.de
# Copyright:: Copyright (c) 2003 Gerhard Spitzlsperger. All rights reserved.
# License:: LGPL see file LICENSE.
# Version 0.21

# changes 0.2:
# * add tilt
# * add BF2
# * sum bugs are fixed

require 'GS_AreaDialog'
require 'GS_FramePlotCanvas'
require 'GS_Style'

require 'fox'
require 'fox/colors'

include Fox

# basic data for the implants
class ImplantData
  
  def initialize
    @idata = Hash.new
    readDataFile( "implant_data.txt" )
  end
  
  # the arrays data contain the implant 
  # description [energy, projected range, standard deviation] 
  
  def getData(species, into, energy)
    data = @idata[species + "_" + into]     
     
    if ( energy > data[-1][0] || energy < data[0][0] )
      Kernel::raise EnergyOutOfRangeException, 
         "energy out of range"
    end

    # if energy has the minimum allowed value return it 
    if data[0][0] == energy
      result = data[0] 
    else  
      i = 0
      while data[i][0] <= energy
        i += 1
      end
    
      # interpolation (now data[i][0] is > energy)  
      ratio = (1.0*energy - data[i-1][0]) / (data[i][0] - data[i-1][0])
  
      rp = data[i-1][1] + ratio * (data[i][1] - data[i-1][1])
      drp = data[i-1][2] + ratio * (data[i][2] - data[i-1][2])
    
       #p [energy, rp , drp]
      result = [energy, rp , drp]
    end
    
    #p result
    return result
  end  
  
  def getSpecies
    ["B", "As", "P", "Sb", "BF2"]
  end
  
  def getSubstrate
    ["Si", "SiN", "SiO2", "PR"]
  end
  
  # the actual implementation of this routine
  # is extremely simple no checking is performed
  # whether the file structure is ok or not
  def readDataFile( filename )
    aFile = File.new(filename)

    species = ""
    substrate = ""
    key = ""
    
    # 0.11
    # add key2 to generate for B equivalent BF2 implants
    key2 = ""
    
    while line = aFile.gets
      if line =~ /^([\d.]+)\s+([\d.]+)\s([\d.]+)/
        #p line
	e = $1
	rp = $2
	rps = $3
	
	@idata[key].push([e.to_f, rp.to_f, rps.to_f])
	#p @@implant_data[key][-1]
	if key =~ /B_/
	  @idata[key2].push([49.0/11.0 * e.to_f, rp.to_f, rps.to_f])
	end  
      elsif line =~ /^IMPLANT:(\w+):(\w+)/
        species = $1
        substrate = $2
	key = species + "_" + substrate
	@idata[key] = Array.new
	if species == "B" 
	  key2 = "BF2_" + substrate 
	  @idata[key2] = Array.new
	else
          key2 = ""	
	end
      end	
    end
  end   
end

# define exceptions
class EnergyOutOfRangeException < RuntimeError
  attr :okToRetry
  def initialize(okToRetry)
    @okToRetry = okToRetry
  end
end

class SingleImplant
  @@iD = ImplantData.new

  attr_reader :species, :energy, :dose, :tilt, :substrate 
  attr_reader :isAcceptor, :isDonor
  attr_reader :cpeak, :rp, :drp
  
  def initialize(species, energy, dose, tilt, substrate) 
    @species = species
    
    if @species == "B" || @species == "In" || @species == "BF2" 
      @isAcceptor = true
      @isDonor = false
    elsif @species == "P" || @species == "As" || @species == "Sb"
      @isAcceptor = false
      @isDonor = true
    else
      @isAcceptor = false
      @isDonor = false   
    end
    
    @energy = energy
    # dose in atoms/cm^2
    @dose = dose
    # definition of the substrate array of [thickness, in substrate]
    @substrate = substrate
    
    # tilt in rad
    @tilt = tilt
    @cos_tilt = Math.cos(@tilt/180 * 3.1415);
    
    # projected range in Si
    @rp = @@iD.getData(@species, "Si", @energy)[1]
    
    # first convert all layer thicknesses to equivalent Si thickness
    @equ_t = @substrate.map{ |item| 
      item[0] *  @rp / @@iD.getData(@species, item[1], @energy)[1]
    }

    # peak concentration in atoms/cm^3
    @drp = @@iD.getData(@species, "Si", @energy)[2]
    @cpeak = dose / (1e-4* @drp * Math.sqrt(2*3.1415))
  end 
  
  # convert real depth to equivalent Si depth
  def realD2siD(x)
    i = 0   # i holds the relevant layer after the looping
    t_Si = 0.0 # accumulated Si thickness 
    t_r = 0.0  # accumulated real thickness
    
    l_Si = @equ_t[0] # accumulated Si thickness per layer 
                     # to decide how to accumulate
    l_r = @substrate[0][0]  # accumulated real thickness    
    
    # 0.11:
    # after this loop t_r, t_Si holds the accumulated thickness of the
    # layers which are totally below depth x, l_Si and l_r holds the
    # thickness of all layers including the last partial layer (where x is inside)    
    while l_r < x
        t_r +=@substrate[i][0] # add the real layer thickness
        t_Si += @equ_t[i] # add the layer Si thickness
    
        i += 1 # we are at least in layer i
	
        l_Si += @equ_t[i] # add the layer Si thickness
	l_r +=@substrate[i][0] # add the real layer thickness
    end  
    
    # now convert the remaining thickness to Si thickness
    rem = x - t_r
    r_Si = @@iD.getData(@species, "Si", @energy)[1]
    
    r_actual_sub = @@iD.getData(@species, @substrate[i][1], @energy)[1]  
    rem_Si = rem *  r_Si / r_actual_sub 
    
    # now the Si depth is
    return t_Si + rem_Si
  end

  
  # calculates the concentration in depth x [um]
  def calc(x)
    x = realD2siD(x) # convert to Si depth first
    
    # 0.11
    x = x / @cos_tilt;    # scale for tilt  
    result = @cpeak * Math.exp(-(x-@rp)**2 / @drp**2) 
        
    return result 
  end  
end

class AddImplantDialog < FXDialogBox

  def initialize(owner, implant_data)
    @iD = implant_data

    super(owner, "Add new Implant", DECOR_TITLE|DECOR_BORDER)

    # Bottom buttons
    buttons = FXHorizontalFrame.new(self,
      LAYOUT_SIDE_BOTTOM|FRAME_NONE|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,
      0, 0, 0, 0, 40, 40, 20, 20)

    # Cancel
    FXButton.new(buttons, "&Cancel", nil, self, ID_CANCEL,
      FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
          
    # Ok
    FXButton.new(buttons, "&Ok", nil, self, ID_ACCEPT,
      FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)

    # Separator
    FXHorizontalSeparator.new(self,
      LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|SEPARATOR_GROOVE)
   
    # content
    matrix = FXMatrix.new(self, 2, MATRIX_BY_COLUMNS)
    
    # dose   
    FXLabel.new(matrix, "dose [ions/cm^2]", nil, LAYOUT_LEFT)
    @dose = FXTextField.new(matrix, 10, nil, 0, TEXTFIELD_REAL|
      LAYOUT_SIDE_BOTTOM|LAYOUT_LEFT|FRAME_SUNKEN|FRAME_THICK)  
    @dose.setText("1.5e14")
    
    # dose   
    FXLabel.new(matrix, "tilt [degree]", nil, LAYOUT_LEFT)
    @tilt = FXTextField.new(matrix, 10, nil, 0, TEXTFIELD_REAL|
      LAYOUT_SIDE_BOTTOM|LAYOUT_LEFT|FRAME_SUNKEN|FRAME_THICK)  
    @tilt.setText("0.0")    
    
    # energy
    FXLabel.new(matrix, "energy [kV]", nil, LAYOUT_LEFT)
    @energy = FXTextField.new(matrix, 10, nil, 0, TEXTFIELD_REAL|
      LAYOUT_SIDE_BOTTOM|LAYOUT_LEFT|FRAME_SUNKEN|FRAME_THICK)  
    @energy.setText("80")
    
    # species   
    FXLabel.new(matrix, "species", nil, LAYOUT_LEFT)  
    
    pane = FXPopup.new(self)
    
    species = @iD.getSpecies
    
    species.each { |item|
      FXOption.new(pane, item, nil, nil, 0, JUSTIFY_HZ_APART|ICON_AFTER_TEXT)
    }
 
    # Option menu
    @species = FXOptionMenu.new(matrix, pane, (FRAME_RAISED|FRAME_THICK|
      JUSTIFY_HZ_APART|ICON_AFTER_TEXT|LAYOUT_CENTER_X|LAYOUT_CENTER_Y))
  end
  
  def getResults
    return [@species.getCurrent, @energy.getText.to_f, 
            @dose.getText.to_f, @tilt.getText.to_f] 
  end
end

class EditLayerDialog < FXDialogBox

  def initialize(owner, implant_data, thickness, material)
    @iD = implant_data

    super(owner, "Edit Layer", DECOR_TITLE|DECOR_BORDER)

    # Bottom buttons
    buttons = FXHorizontalFrame.new(self,
      LAYOUT_SIDE_BOTTOM|FRAME_NONE|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,
      0, 0, 0, 0, 40, 40, 20, 20)

    # Cancel
    FXButton.new(buttons, "&Cancel", nil, self, ID_CANCEL,
      FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
          
    # Ok
    FXButton.new(buttons, "&Ok", nil, self, ID_ACCEPT,
      FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)

    # Separator
    FXHorizontalSeparator.new(self,
      LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|SEPARATOR_GROOVE)
   
    # content
    matrix = FXMatrix.new(self, 2, MATRIX_BY_COLUMNS)
    
    FXLabel.new(matrix, "thickness [um]", nil, LAYOUT_LEFT)
    @th = FXTextField.new(matrix, 10, nil, 0, TEXTFIELD_REAL|
      LAYOUT_SIDE_BOTTOM|LAYOUT_LEFT|FRAME_SUNKEN|FRAME_THICK)  
    @th.setText(thickness.to_s)
    
    # species   
    FXLabel.new(matrix, "material", nil, LAYOUT_LEFT)  
    
    pane = FXPopup.new(self)
    
    s = @iD.getSubstrate
    
    # to move the preset material to the top position it must be top
    # in the array
    s.delete(material)
    s.unshift(material)
 
    s.each { |item|
      FXOption.new(pane, item, nil, nil, 0, JUSTIFY_HZ_APART|ICON_AFTER_TEXT)
    }
 
    # Option menu
    @layer = FXOptionMenu.new(matrix, pane, (FRAME_RAISED|FRAME_THICK|
      JUSTIFY_HZ_APART|ICON_AFTER_TEXT|LAYOUT_CENTER_X|LAYOUT_CENTER_Y))
  end
  
  def getResults
    return [@th.getText.to_f, @layer.getCurrent.to_s ] 
  end
end


class EditSubstrateDialog < FXDialogBox
  attr_reader :finishButton

  def initialize(owner, implant_data, substrate)
    @iD = implant_data
    @substrate = substrate

    super(owner, "Edit Substrate", DECOR_TITLE|DECOR_BORDER)

    # Bottom buttons
    buttons = FXHorizontalFrame.new(self,
      LAYOUT_SIDE_BOTTOM|FRAME_NONE|LAYOUT_FILL_X|PACK_UNIFORM_WIDTH,
      0, 0, 0, 0, 40, 40, 20, 20)

    # Cancel
    FXButton.new(buttons, "&Cancel", nil, self, ID_CANCEL,
      FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
          
    # Ok
    FXButton.new(buttons, "&Ok", nil, self, ID_ACCEPT,
      FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)

    # Separator
    FXHorizontalSeparator.new(self,
      LAYOUT_SIDE_BOTTOM|LAYOUT_FILL_X|SEPARATOR_GROOVE)
   
    # content
    create_substrate
  end
  
  def create_substrate
    @matrix.destroy if @matrix
    @matrix = FXMatrix.new(self, 5, MATRIX_BY_COLUMNS)
    
    @substrate.each_index{ |i|
      FXLabel.new(@matrix, @substrate[i][0].to_s, nil, LAYOUT_LEFT)
      FXLabel.new(@matrix, @substrate[i][1], nil, LAYOUT_LEFT)
    
      e = FXButton.new(@matrix, "Edit", nil, self, 0,
      FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
      e.connect(SEL_COMMAND) { |sender, sel, event| 
        edit_layer(i)
      }

      b = FXButton.new(@matrix, "Insert", nil, self, 0,
      FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
      b.connect(SEL_COMMAND) { |sender, sel, event| 
        insert_layer(i)
      }
      
      d = FXButton.new(@matrix, "Delete", nil, self, 0,
      FRAME_RAISED|FRAME_THICK|LAYOUT_RIGHT|LAYOUT_CENTER_Y)
      d.connect(SEL_COMMAND) { |sender, sel, event| 
        delete_layer(i)
      }
    } 
     
  end  
  
  def delete_layer(pos)
    @substrate.delete_at(pos)
    @finishButton = "d"
    onCmdAccept(self, ID_ACCEPT, nil)
  end  
  
  def getResults
    return @substrate 
  end
  
  def edit_layer(pos)
    @finishButton = "e"
    d = EditLayerDialog.new(self, @iD, @substrate[pos][0],
                            @substrate[pos][1])  

    begin  # prepare exception handling
      if d.execute == 1
        r = d.getResults
	@substrate[pos] = r
	p @substrate
      end  
    end	# end exception  
    
    # update 
    onCmdAccept(self, ID_ACCEPT, nil)
  end
  
  def insert_layer(pos)
    @substrate[pos, 1] = [[1.0, "Si"], @substrate[pos]]
    @finishButton = "i"
    onCmdAccept(self, ID_ACCEPT, nil)
  end  
end


class CanvasWindow < FXMainWindow  

  def initialize(app)
    @init_XSize = 900
    @init_YSize = 600
   
    # Call base class initializer first
    super(app, "Implant Profile", nil, nil, DECOR_ALL,
      0, 0, @init_XSize, @init_YSize)

    # make the implant data available 
    @@iD = ImplantData.new
    
    # load some icons  
    killicon = getIcon("kill.png")
    newicon = getIcon("filenew.png")     
     
    # Make menu bar
    menubar = FXMenubar.new(self, LAYOUT_FILL_X)
    
    # the file menu
    filemenu = FXMenuPane.new(self)  
    FXMenuTitle.new(menubar, "&File", nil, filemenu)
 
    resetCmd = FXMenuCommand.new(filemenu, "&Reset")
    resetCmd.connect(SEL_COMMAND, method(:onCmdResetImplants))  
    
    FXMenuSeparator.new(filemenu)
    FXMenuCommand.new(filemenu, "&Quit\tCtl-Q", nil,
      getApp(), FXApp::ID_QUIT)
    
    # the implant menu
    implantmenu = FXMenuPane.new(self)
    FXMenuTitle.new(menubar, "&Implant", nil, implantmenu)   
    setImplantCmd = FXMenuCommand.new(implantmenu, "&Add Implant")
    setImplantCmd.connect(SEL_COMMAND, method(:onCmdSetImplant))   

    setImplantCmd = FXMenuCommand.new(implantmenu, "&Edit Substrate")
    setImplantCmd.connect(SEL_COMMAND, method(:onCmdEditSubstrate))   

    # the graphic menu
    graphicmenu = FXMenuPane.new(self)
    FXMenuTitle.new(menubar, "&Graphics", nil, graphicmenu)   
    scaleAxisCmd = FXMenuCommand.new(graphicmenu, "&Scale Axis")
    scaleAxisCmd.connect(SEL_COMMAND, method(:onCmdShowAreaDialog))
     
    # the help menu
    helpmenu = FXMenuPane.new(self)
    FXMenuTitle.new(menubar, "&Help", nil, helpmenu, LAYOUT_RIGHT)
    FXMenuCommand.new(helpmenu, "&About ...").connect(SEL_COMMAND) {
      FXMessageBox.information(self, MBOX_OK, "About Implant Profile",
        "A really simple program to plot implant profiles based on \n" +
        "Gaussian distributions (www.gs68.de)\n\n" +
	"Copyright 2003 gerhard.spitzlsperger@gs68.de")
    }
    
    # contains the canvas
    @canvasFrame = FXVerticalFrame.new(self,
      FRAME_SUNKEN|LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 0, 0, 0, 0)

    # Place a toolbar above the canvas
    toolbar = FXToolbar.new(@canvasFrame, (LAYOUT_SIDE_TOP|
      PACK_UNIFORM_WIDTH|PACK_UNIFORM_HEIGHT|FRAME_RAISED|LAYOUT_FILL_X))
 
    FXButton.new(toolbar, "Exit\tExit", killicon, app, FXApp::ID_QUIT,
      ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED)     
 
    nb = FXButton.new(toolbar, "Reset\tReset", newicon, nil, 0,
      ICON_ABOVE_TEXT|BUTTON_TOOLBAR|FRAME_RAISED)     
    nb.connect(SEL_COMMAND) { |sender, sel, event|  resetImplants }
    
    # Horizontal divider line
    FXHorizontalSeparator.new(@canvasFrame, SEPARATOR_GROOVE|LAYOUT_FILL_X)

    # contains active area
    @activeFrame = FXSplitter.new(@canvasFrame, (LAYOUT_SIDE_TOP|LAYOUT_FILL_X|
      LAYOUT_FILL_Y|SPLITTER_REVERSED|SPLITTER_TRACKING))

    # some global settings
    @implantSummary = [["summary_total_dose", "total dose"],
                       ["summary_total_donor", "total donor dose"],
		       ["summary_total_acceptor", "total acceptor dose"],
		       ["summary_eff_donor", "eff. donor dose"],
		       ["summary_eff_acceptor", "eff. acceptor dose"]]

    # some styles for the various implants ...
    @actualImplant = 0
    @implantStyles = Array.new
    
    @implantStyles[0] = GS_Style.new
    
    for i in 1 .. 5
      @implantStyles[i] = GS_Style.new
      @implantStyles[i].lineStyle = LINE_ONOFF_DASH
      @implantStyles[i].dashPattern = [1 + i*3, 1 + i*3]
    end 
    
    for i in 1 .. 5
      @implantStyles[i+5] = GS_Style.new
      @implantStyles[i+5].lineStyle = LINE_DOUBLE_DASH
      @implantStyles[i+5].dashPattern = [1 + i*3, 1 + i*3]
    end 
 
    # initial world coordinates for the drawing canvas
    x_min = 0.0
    x_max = 0.1
    y_min = 12.0  # already logarithmic  
    y_max = 17.0  # already logarithmic
     
    # Drawing canvas
    @canvas = GS_FramePlotCanvas.new(@activeFrame, nil, 0, (FRAME_SUNKEN|FRAME_THICK|
      LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT), 
      [x_min, y_min, x_max, y_max], "linlog" )
     @canvas.title = "Dopant Concentration (ions/cm^3) vs. Depth (um)"
      
    @canvas.connect(SEL_PAINT) { |sender, sel, event|
      @canvas.draw
    }  
      
    # define the substrate
    # the setSubstrate call relies on @canvas being defined    
    @substrate = [[0.1, "PR"], [0.025, "SiN"],[1.0/0.0, "Si"]] 
    setSubstrate(@substrate)   
       
       
    @legendFrame = FXVerticalFrame.new(@activeFrame,
      FRAME_SUNKEN|LAYOUT_MIN_WIDTH|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT,
      0, 0, 0, 0, 0, 0, 0, 0) 
    @legendFrame.setWidth( @init_XSize * 0.29 )  
    
    vv = FXGroupBox.new(@legendFrame, "Summary",
      LAYOUT_SIDE_TOP|FRAME_GROOVE|LAYOUT_FILL_X, 0, 0, 0, 0)
      
    @cb_summary = Hash.new      
    
    @implantSummary.each{ |item|
      @cb_summary[item[0]] = FXCheckButton.new(vv, item[1], nil, 0,
      ICON_BEFORE_TEXT|LAYOUT_SIDE_TOP)
    
      @cb_summary[item[0]].connect(SEL_COMMAND) { |sender, sel, checked|
        sumUpImplants
      }	
    }
    
    
    # the description canvas
    @legend = GS_PlotCanvas.new(@legendFrame, nil, 0, (FRAME_SUNKEN|FRAME_THICK|
      LAYOUT_FILL_X|LAYOUT_FILL_Y|LAYOUT_TOP|LAYOUT_LEFT), 
      [0.0, 0.0, 1.0, 1.0])
    @legend.setMargin(0,0,0,0)	
    	
    @legend.connect(SEL_PAINT) { |sender, sel, event|
      @legend.draw
    }    
  end

  # rescales the plot after getting new xmax and ymax
  def rescale(xmax, ymax)
    # check whether a new setting for the plot is mandatory
    temp = Math.log10(ymax)  # we are plotting on a logarithmic plot ...
				    # axis limits are calculated in exponents
    y_max =  ( temp > @canvas.y_max ) ? temp : @canvas.y_max
    x_max =  ( xmax > @canvas.x_max ) ? xmax : @canvas.x_max
    
    begin  # prepare exception handling
      @canvas.setArea( [@canvas.x_min, x_max, 
                        @canvas.y_min, y_max.ceil] )
    end # end exception 
  end

  def resetImplants
    @actualImplant = 0

    @canvas.items.delete_if{ |item| item[0] =~ /implant/ }
    
    sumUpImplants
  end  

  def setImplant(implant)
    # draw the implant from peak till 5 sigma
    xmax = (implant.rp + 5*implant.drp) > @canvas.x_max ?
                (implant.rp + 5*implant.drp) : @canvas.x_max

    gf = GS_Function.new( 0.0, xmax, 399, implant)
    gf.baseStyle = @implantStyles[@actualImplant]
    
    @actualImplant = @actualImplant + 1
    
    type = implant.isAcceptor ? "acceptor" : "donor"

    @canvas.append_item( "implant" + type + @actualImplant.to_s +
                         "_" + implant.species + 
                         "_" + implant.energy.to_s + "_" +
			 implant.dose.to_s + "_" +
			 implant.tilt.to_s , gf )
    rescale( xmax, gf.getMax() )
    
    sumUpImplants
  end
  
  def setSubstrate(substrate)
    # after set new substrate all implants are invalid
    # and substrate need to be drawn again
    resetImplants if @actualImplant != 0
    @canvas.items.delete_if{ |item| item[0] =~ /layer/ }
    
    @substrate = substrate
    
    # init the depth
    d = 0.0
    @substrate.each{ |item| 
    
      # exclude the last one
      if item[0].infinite? == nil 
        d = d + item[0]    
	line = GS_LogAxis.new("GS_LogYAxis_Autoscale", 
	                        d, @canvas.y_min, d, @canvas.y_max)
        line.ticks_and_numbers = false					
        @canvas.append_item( "layer" + d.to_s, line )
      end	
    }
  end
  
  def sumUpImplants
    # define the settings
    settings = [["summary_total_dose",     '^implant',         FXColor::Green],
                ["summary_total_acceptor", '^implantacceptor', FXColor::Yellow],
		["summary_total_donor",    '^implantdonor',    FXColor::Pink]]
  
    settings.each{ |item| 
      # delete the old summary
      @canvas.items.delete(item[0])
      
      rxp =  Regexp.new(item[1])
      temp = @canvas.items.select{ |i| rxp.match(i[0]) }  
   
      # add the summary if there are related implants 
      if temp.nitems != 0
        fct = GS_SumFunctionContainer.new
   
        temp.each{ |i| fct.append_item(i[0], i[1]) }
   
        gf = GS_Function.new( 0.0, @canvas.x_max, 89, fct)
        gf.baseStyle = GS_Style.new
        gf.baseStyle.foreColour = item[2]
	
	@cb_summary[item[0]].getCheck == true ? gf.isVisible = true : gf.isVisible = false
        @canvas.append_item( item[0], gf )
    
        rescale( @canvas.x_max, gf.getMax() )
      end	
 
    } 
      
    # are there any implants
    temp = @canvas.items.select{ |i| /^implant/.match(i[0])}
    if temp.nitems != 0 and @canvas.items["summary_total_acceptor"] !~ /not existing/ and @canvas.items["summary_total_donor"] !~ /not existing/
      # the effective acceptors
      @canvas.items.delete("summary_eff_acceptor")
      eff_acceptors = 
        GS_DiffFunctionContainer.new(
	    @canvas.items["summary_total_acceptor"],
	    @canvas.items["summary_total_donor"])
    
      gf = GS_Function.new( 0.0, @canvas.x_max, 89, eff_acceptors)
      gf.baseStyle = GS_Style.new
      gf.baseStyle.foreColour = FXColor::Red
      @cb_summary["summary_eff_acceptor"].getCheck == true ? gf.isVisible = true : gf.isVisible = false
      @canvas.append_item( "summary_eff_acceptor", gf )
      rescale( @canvas.x_max, gf.getMax() )
      
    
      # the effective donors
      @canvas.items.delete("summary_eff_donor")
      eff_acceptors = 
        GS_DiffFunctionContainer.new(
	    @canvas.items["summary_total_donor"],
	    @canvas.items["summary_total_acceptor"])
    
      gf = GS_Function.new( 0.0, @canvas.x_max, 89, eff_acceptors)
      gf.baseStyle = GS_Style.new
      gf.baseStyle.foreColour = FXColor::Blue
      @cb_summary["summary_eff_donor"].getCheck == true ? gf.isVisible = true : gf.isVisible = false
      @canvas.append_item( "summary_eff_donor", gf )
      rescale( @canvas.x_max, gf.getMax() )
    else
      @canvas.items.delete("summary_eff_acceptor")
      @canvas.items.delete("summary_eff_donor")
    end
    
    @canvas.draw
    updateLegend
  end
  
  def updateLegend

    @legend.items.clear
  
    t = @canvas.items.keys.select { |item| 
           (@canvas.items[item].isVisible == true) && (item =~ /^implant/  || item =~ /^summary/  )
	}

    y = 0.95
    t.each{ |item|
      l = GS_Line.new(0.05, y, 0.2, y)
      l.baseStyle = @canvas.items[item].baseStyle
      @legend.append_item( item + "_line", l )
   
      tt = /^.+?_(.+)$/.match(item)   # +? means match only till first occurence
      d = GS_Text.new(0.25, y, tt[1])
      @legend.append_item( item + "_text", d )
  
      y = y - 0.08
    }
  
    @legend.draw
  end
  
  def onCmdEditSubstrate(sender, sel, ptr)
    d = EditSubstrateDialog.new(self, @@iD, @substrate)

    begin  # prepare exception handling
      while d.execute == 1
        @substrate = d.getResults
	p @substrate

	unless d.finishButton
	  setSubstrate(@substrate)
	  break
	end  
	
	d = EditSubstrateDialog.new(self, @@iD, @substrate)
      end  
    end	# end exception
    
  end

  def onCmdResetImplants(sender, sel, ptr)
    resetImplants
  end
  
  def onCmdSetImplant(sender, sel, ptr)
    d = AddImplantDialog.new(self, @@iD)
    
    begin  # prepare exception handling
      if d.execute == 1
        r = d.getResults
	
        implant1 = SingleImplant.new(r[0].getText, r[1], 
            r[2], r[3], @substrate)
        setImplant(implant1)	
      end  
    rescue EnergyOutOfRangeException
      FXMessageBox.warning(self, MBOX_OK, "Energy out of Range!", 
      "Please insert a correct energy or cancel.")
      retry
    end	# end exception
  end  
  
  def onCmdShowAreaDialog(sender, sel, ptr)
    d = GS_AreaDialog.new(self, @canvas.getArea)

    begin  # prepare exception handling
      if d.execute == 1
        area = d.getResults
        @canvas.setArea(area)
	@canvas.draw
      end  
    rescue DrawAreaException
      FXMessageBox.warning(self, MBOX_OK, "Wrong Area Setting", 
          "Please insert a correct area or cancel.")
      retry
    end	# end exception
  end  

  # function to construct a GIF icon
  def getIcon(filename)
    FXPNGIcon.new(getApp(), File.open(filename, "rb").read)
  end
  
  # Create and show the main window
  def create
    super                  # Create the windows
    show(PLACEMENT_SCREEN) # Make the main window appear
  end
end

def run
  application = FXApp.new('SIMPLANT', 'SIMPLANT')
  application.init(ARGV)
  
  # Construct the main window
  s = CanvasWindow.new(application)
  
  # Create the application
  application.create
  
  # Run the application
  application.run
end



# run the application
run

